/*
 * Copyright (C) 2003-2005 Bjrn-Ove Heimsund
 * 
 * This file is part of MT.
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package mt.test;

import mt.Matrix;
import mt.fact.OrthogonalDecomposition;
import mt.fact.QRComputer;

/**
 * QR test
 */
public class QRTest extends OrthogonalTest {

    public QRTest(String arg0) {
        super(arg0);
    }

    public void testStaticFactorize() {
        assertEquals(A, QRComputer.factorize(A));
    }

    public void testRepeatStaticFactorize() {
        assertEquals(A, QRComputer.factorize(A));
        assertEquals(A, QRComputer.factorize(A));
    }

    public void testFactor() {
        QRComputer c = new QRComputer(A.numRows(), A.numColumns());
        assertEquals(A, c.factor(A.copy()));
    }

    public void testRepeatFactor() {
        QRComputer c = new QRComputer(A.numRows(), A.numColumns());
        OrthogonalDecomposition od = c.factor(A.copy());
        assertEquals(A, od);
        od = c.factor(A.copy());
        assertEquals(A, od);
    }

    public void testStaticFactorizeNonSquare() {
        assertEquals(Ar, QRComputer.factorize(Ar));
    }

    public void testRepeatStaticFactorizeNonSquare() {
        assertEquals(Ar, QRComputer.factorize(Ar));
        assertEquals(Ar, QRComputer.factorize(Ar));
    }

    public void testFactorNonSquare() {
        QRComputer c = new QRComputer(Ar.numRows(), Ar.numColumns());
        assertEquals(Ar, c.factor(Ar.copy()));
    }

    public void testRepeatFactorNonSquare() {
        QRComputer c = new QRComputer(Ar.numRows(), Ar.numColumns());
        OrthogonalDecomposition od = c.factor(Ar.copy());
        assertEquals(Ar, od);
        od = c.factor(Ar.copy());
        assertEquals(Ar, od);
    }

    private void assertEquals(Matrix A, OrthogonalDecomposition od) {
        assertEquals(A, od.getQ().mult(od.getR(), A.copy().zero()));
    }

}
